#coding=utf-8

"""
读写文件再简化
"""
def fread(fp, mode='rb'):
    with open(fp, mode) as f:
        return f.read()

pass
read=fread
def freads(fp, mode = 'rb', size=1024*1024):
    with open(fp, mode) as f:
        while True:
            bs = f.read(size)
            if len(bs)==0:
                break
            yield bs

pass
reads=freads

def fwrite(fp, ct, mode = 'wb'):
    with open(fp, mode) as f:
        f.write(ct)

pass
write = fwrite
def fwrites(fp, cts, mode = 'wb'):
    with open(fp, mode) as f:
        for ct in cts:
            f.write(ct)

pass
writes = fwrites



def get(obj, key, default=None):
    if key not in obj:
        return default
    return obj[key]

pass
def g(obj, **maps):
    rst = []
    for k in maps:
        v = maps[k]
        if k in obj:
            v = obj[k]
        rst.append(v)
    if len(rst)==1:
        rst = rst[0]
    return rst

pass
def s(obj, **maps):
    for k in maps:
        v = maps[k]
        obj[k] = v

pass
def sets(maps, keys, val):
    if type(keys) != list:
        keys = [keys]
    for key in keys[:-1]:
        if key not in maps:
            maps[key] = {}
        maps = maps[key]
    maps[keys[-1]] = val

pass
def gets(maps, keys, default = None):
    if type(keys) != list:
        keys = [keys]
    for key in keys:
        if key not in maps:
            return default
        maps = maps[key]
    return maps

pass
def removes(maps, keys):
    if type(keys) != list:
        keys = [keys]
    arr = []
    for key in keys:
        if key not in maps:
            break
        arr.append([maps, key])
        maps = maps[key]
    arr.reverse()
    first=1
    for maps,key in arr:
        if first or len(maps[key])==0:
            del maps[key]
        first=0
    return None

pass
def l2m(arr, **maps):
    rst = {}
    i = 0
    for k in maps:
        if i<len(arr):
            val = arr[i]
        else:
            val = maps[k]
        rst[k] = val
        i+=1
    return rst

pass

def deep_update(target, src, replace=1):
    """
        dict深层更新，src[key]是dict就深入更新，否则:
            src有而maps没有就替换，否则：
                replace=1就替换
    """
    for k in src:
        val = src[k]
        if k not in target:
            target[k] = val
            continue
        mval = target[k]
        if type(mval) == dict and type(val)==dict:
            update_maps(mval, val, replace)
        else:
            if replace:
                target[k] = val

pass


#coding=utf-8
from buildz import xf, pyz
from buildz.xf import g as xg
import json
class Base:
    def update_maps(self, maps, src):
        xf.deep_update(maps, src)
    def __init__(self, *args, **maps):
        self.init(*args, **maps)
    def init(self, *args, **maps):
        pass
    def __call__(self, *args, **maps):
        return self.deal(*args, **maps)
    def deal(self, *args, **maps):
        return None

pass
class EncapeData(Base):
    """
        包含data id对应的配置，配置文件id，配置文件对象
        [object.test, call, ]
    """
    def __init__(self, data, conf, local = False, type = None, src = None, info = None):
        self.data = data
        self.sid = conf.id
        self.src = src
        self.conf = conf
        self.confs = conf.confs
        self.local = local
        if type is None:
            type = conf.confs.get_data_type(data, 0, conf.default_type())
        self.type = type
        self.info = info

pass



from .. import base
from .. import item
from .. import exp
from . import lr
class ListDeal(lr.LRDeal):
    """
        分隔符，有分隔符后将缓存的数据当作字符串
    """
    def init(self, left, right):
        super().init(left, right, 'list')
    def build(self, arr):
        rst = []
        for _item in arr:
            if not _item.is_val:
                raise Exception("error in list:"+_item)
            rst.append(_item.val)
        return item.Item(rst, type='list', is_val = 1)

pass


from .. import base


class PrevNextDeal(base.BaseDeal):
    def labels(self):
        return ['']
    def types(self):
        return [""]
    def build(self, obj):
        obj.is_val = 1
        return obj
    """
        读取下一个字符放缓存里，应放最低优先级
    """
    def deal(self, buffer, arr, mg):
        c = buffer.read_cache(1)
        if len(c)==0:
            return False
        return True

pass